@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# $Id: wheel_run.pm 2671 2009-09-04 07:49:18Z rcaputo $
+# vim: ts=2 sw=2 expandtab
 
 use strict;
 use lib qw(./mylib ../mylib);
@@ -23,13 +23,13 @@ BEGIN {
     if ($@) {
       $error = "Win32::Console is required on $^O - try ActivePerl";
     }
-    elsif (exists $INC{"Tk.pm"}) {
+    elsif (exists $INC{"Tk.pm"} and not $ENV{POE_DANTIC}) {
       $error = "$^O with Tk seems to hang on this test";
     }
-    elsif (exists $INC{"Event.pm"}) {
+    elsif (exists $INC{"Event.pm"} and not $ENV{POE_DANTIC}) {
       $error = "$^O\'s fork() emulation breaks Event";
     }
-    else {
+    elsif (not $ENV{POE_DANTIC}) {
       $error = "Signal handling on $^O is too fragile - Perl crashes";
     }
 
@@ -43,15 +43,27 @@ BEGIN {
 
   sub STD_TEST_COUNT () { 8 }
 
-  plan tests => 4 + 15 + 8 + 8 + 8 * STD_TEST_COUNT;
+  plan tests =>
+    4 +
+    15 +
+    8 +
+    8 +
+    4 + #Silent/Open FD tests
+    8 * STD_TEST_COUNT;
 }
 
 # Turn on extra debugging output within this test program.
 sub DEBUG () { 0 }
 
 sub POE::Kernel::ASSERT_DEFAULT () { 1 }
-sub POE::Kernel::TRACE_DEFAULT  () { 1 }
-sub POE::Kernel::TRACE_FILENAME () { "./test-output.err" }
+
+BEGIN {
+  package POE::Kernel;
+{
+  $POE::Kernel::VERSION = '1.351';
+}
+  use constant TRACE_DEFAULT => exists($INC{'Devel/Cover.pm'});
+}
 
 use POE qw(Wheel::Run Filter::Line);
 
@@ -151,6 +163,7 @@ my $shutdown_program = sub {
           $_[KERNEL]->delay(check_timeout => undef);
           $_[KERNEL]->alias_remove("timeout");
         },
+        _stop => sub { }, # Pacify assertions.
       },
     );
     return $sess->ID;
@@ -176,6 +189,9 @@ my $shutdown_program = sub {
 
 { # {{{ a proxy around POE::Filter::Line that doesn't support get_one
   package My::LineFilter;
+{
+  $My::LineFilter::VERSION = '1.351';
+}
   sub new {
     my $class = shift;
     return bless [ POE::Filter::Line->new(@_) ], $class;
@@ -238,9 +254,6 @@ sub main_start {
 
   $heap->{label} = $label;
 
-  # Handle SIGCHLD
-  $kernel->sig(CHLD => "sigchld");
-
   # Sometimes use a filter without get_one support
   my $filter_class = "POE::Filter::Line";
   if ($main_counter++ % 2) {
@@ -274,9 +287,11 @@ sub main_start {
     CloseEvent  => 'close',
   );
 
+  $_[KERNEL]->sig_child($heap->{wheel}->PID, "sigchld");
+
   # start the test statemachine
   $heap->{expected} = [@$expected];
-  &main_perform_state;
+  &main_perform_state; # Deliberately passing @_ through.
 
   $heap->{flushes_expected} = scalar(
     grep { (!ref $_->[0]) and defined($_->[1]) } @$expected
@@ -400,14 +415,6 @@ sub create_constructor_session {
   my $sess = POE::Session->create(
     inline_states => {
       _start => sub {
-        eval {
-          POE::Wheel::Run->new(
-            Program => sub { 1; },
-            ErrorEvent => 'error_event',
-          );
-        };
-        ok(!(!$@), "new: at least one io event");
-
         eval {
           POE::Wheel::Run->new(
             Program => sub { 1; },
@@ -443,9 +450,19 @@ sub create_constructor_session {
           );
         };
         ok(!(!$@), "new: Program is needed");
-
+        
+        eval {
+          POE::Wheel::Run->new(
+            Program => sub { 0 },
+            StdoutEvent => "stdout_nonexistent",
+            RedirectStdout => "/non/existent"
+          );
+        };
+        ok(!(!$@), "new: *Event and Redirect* are mutually exclusive");
+        
         timeout_poke();
       },
+      _stop => sub { }, # Pacify assertions.
     },
   );
 
@@ -526,6 +543,97 @@ SKIP: {
   );
 }
 
+sub silent_start {
+  
+  pipe my ($stdout_read,$stdout_write);
+  pipe my ($stdin_read, $stdin_write);
+  my $wheel = POE::Wheel::Run->new(
+    Program => sub {
+      eval "print STDOUT 'CHILD:'";
+      eval 'my $input = <STDIN>; chomp($input); print STDERR $input;';
+      eval 'print STDERR "CHILD:";';
+      exit(0);
+    },
+    
+    RedirectOutput => $stdout_write,
+    RedirectStdin  => $stdin_read
+    
+  );
+  
+  select $stdin_write; $|=1;
+  
+  $_[HEAP]->{silent_fdes} = [$stdout_read, $stdin_write];
+  $_[HEAP]->{silent_wheel} = $wheel;
+  
+  print $stdin_write "PARENT:\n";
+  $poe_kernel->select_read($stdout_read, 'silent_got_stdout');
+  $poe_kernel->sig_child($wheel->PID, 'silent_sigchld');
+    
+  my $no_stdio = POE::Wheel::Run->new(
+    Program => \&note
+  );
+  $poe_kernel->sig_child($no_stdio->PID, 'silent_sigchld');
+  
+  ok(!($no_stdio->[ $no_stdio->HANDLE_STDOUT ] ||
+         $no_stdio->[ $no_stdio->HANDLE_STDERR ]),
+         "stdio/standard output handles closed without events");
+  
+  ok($no_stdio->[ $no_stdio->HANDLE_STDIN ],
+     "stdio discard/STDIN still alive");
+
+  $no_stdio = POE::Wheel::Run->new(
+    Program => \&note,
+    NoStdin => 1
+  );
+  
+  $poe_kernel->sig_child($no_stdio->PID, 'silent_sigchld');
+  ok(!($no_stdio->[ $no_stdio->HANDLE_STDIN ]),
+         "stdio/discarded STDIN with NoStdin");
+}
+
+sub _silent_check_common {
+  my ($fh,$re,$desc) = @_;
+  sysread($fh, my $buf = "", 8192);
+  like($buf, $re, $desc);
+  $poe_kernel->select_read($fh);
+  close($fh);
+}
+
+sub silent_got_stdout {
+  _silent_check_common(
+    $_[ARG0],
+    qr/CHILD:PARENT:CHILD:/,
+    "stdio/redirection"
+  );
+}
+
+sub silent_fd_status {
+  _silent_check_common(
+    $_[ARG0],
+    qr/STDERR:-1,STDIN:-1,STDOUT:-1/,
+    "stdio closed in child");
+}
+
+sub silent_sigchld {
+  #dummy..
+}
+
+
+
+sub silent_test {
+  #Use pipes here for redirection.
+  my $sess = POE::Session->create(
+    inline_states => {
+      _start => \&silent_start,
+      _stop => sub { },
+      #_stop => sub  { note "Stopped!" },
+      silent_got_stdout => \&silent_got_stdout,
+      silent_fd_status  => \&silent_fd_status,
+      silent_sigchld    => \&silent_sigchld
+    }
+  );
+}
+
 for my $chld_program (@chld_programs) {
   my ($chld_name, $chld_code) = @$chld_program;
 
@@ -574,6 +682,8 @@ for my $chld_program (@chld_programs) {
 }
 # }}}
 
+silent_test();
+
 $poe_kernel->run;
 
 1;
