@@ -1,31 +1,56 @@
 #!/usr/bin/perl -w
-# $Id: wheel_sf_ipv6.pm 2672 2009-09-05 17:56:52Z rcaputo $
+# vim: ts=2 sw=2 expandtab
 
 # Exercises Client and Server TCP components, which exercise
 # SocketFactory in AF_INET6 mode.
 
 use strict;
 use lib qw(./mylib ../mylib);
-use Socket;
+
+BEGIN {
+  # under perl-5.6.2 the warning "leaks" from the eval, while newer versions don't...
+  # it's due to Exporter.pm behaving differently, so we have to shut it up
+  no warnings 'redefine';
+  require Carp;
+  local *Carp::carp = sub { die @_ };
+  eval { require Socket; Socket->import('AF_INET6') };
+  if ($@) {
+    eval { require Socket6; Socket6->import('AF_INET6') };
+    if ($@) {
+      print "1..0 # Skip Cannot find AF_INET6 support in Socket or Socket6.\n";
+      CORE::exit();
+    }
+  }
+}
+
+# Second BEGIN block so that AF_INET6 is defined before this code is
+# compiled.
 
 BEGIN {
   my $error;
 
-  eval 'use Socket6 ()';
-  if ( length($@) or not exists($INC{"Socket6.pm"}) ) {
-    $error = "Socket6 is needed for IPv6 tests";
+  eval 'use Socket::GetAddrInfo qw(:newapi getaddrinfo getnameinfo NI_NUMERICHOST NI_NUMERICSERV)';
+  if ($@) {
+    $error = "Socket::GetAddrInfo is needed for IPv6 tests";
   }
   elsif ($^O eq "cygwin") {
-    $error = "IPv6 is not available on Cygwin, even if Socket6 is installed";
+    $error = (
+      "IPv6 isn't available on Cygwin, even with Socket::GetAddrInfo installed"
+    );
   }
   else {
     my $addr;
-    eval { $addr = Socket6::inet_pton(&Socket6::AF_INET6, "::1") };
+    eval {
+      my ($error, @addr) = getaddrinfo(
+        "localhost", 80, { family => AF_INET6 }
+      );
+      $addr = $addr[0]{addr} if @addr;
+    };
     if ($@) {
-      $error = "AF_INET6 not provided by Socket6.pm ... can't test this";
+      $error = "error resolving localhost for IPv6: $@";
     }
     elsif (!defined $addr) {
-      $error = "IPv6 tests require a configured localhost address ('::1')";
+      $error = "IPv6 tests require a configured IPv6 localhost address";
     }
     elsif (!-f 'run_network_tests') {
       $error = "Network access (and permission) required to run this test";
@@ -41,8 +66,14 @@ BEGIN {
 }
 
 sub POE::Kernel::ASSERT_DEFAULT () { 1 }
-sub POE::Kernel::TRACE_DEFAULT  () { 1 }
-sub POE::Kernel::TRACE_FILENAME () { "./test-output.err" }
+
+BEGIN {
+  package POE::Kernel;
+{
+  $POE::Kernel::VERSION = '1.351';
+}
+  use constant TRACE_DEFAULT => exists($INC{'Devel/Cover.pm'});
+}
 
 use POE qw( Component::Client::TCP Component::Server::TCP );
 
@@ -60,7 +91,7 @@ diag( "packets across your localhost interface." );
 POE::Component::Server::TCP->new(
   Port               => 0,
   Address            => '::1',
-  Domain             => Socket6::AF_INET6,
+  Domain             => AF_INET6,
   Alias              => 'server',
   ClientConnected    => \&server_got_connect,
   ClientInput        => \&server_got_input,
@@ -69,9 +100,19 @@ POE::Component::Server::TCP->new(
   Error              => \&server_got_error,
   ClientError        => sub { }, # Hush a warning.
   Started            => sub {
-    $tcp_server_port = (
-      Socket6::unpack_sockaddr_in6($_[HEAP]->{listener}->getsockname())
-    )[0];
+    eval {
+      my $socket_name = $_[HEAP]{listener}->getsockname();
+      (my ($err, $host), $tcp_server_port) = getnameinfo(
+        $socket_name, NI_NUMERICHOST | NI_NUMERICSERV
+      );
+    };
+    if (!$tcp_server_port || $@) {
+      $tcp_server_port = undef;
+      SKIP: {
+        my $errstr = @$ || 'server port undefined';
+        skip "AF_INET6 probably not supported or configured ($errstr)", 2;
+      }
+    }
   },
 );
 
@@ -111,17 +152,19 @@ sub server_got_error {
 ###############################################################################
 # Start the TCP client.
 
-POE::Component::Client::TCP->new(
-  RemoteAddress => '::1',
-  RemotePort    => $tcp_server_port,
-  Domain        => Socket6::AF_INET6,
-  BindAddress   => '::1',
-  Connected     => \&client_got_connect,
-  ServerInput   => \&client_got_input,
-  ServerFlushed => \&client_got_flush,
-  Disconnected  => \&client_got_disconnect,
-  ConnectError  => \&client_got_connect_error,
-);
+if ($tcp_server_port) {
+  POE::Component::Client::TCP->new(
+    RemoteAddress => '::1',
+    RemotePort    => $tcp_server_port,
+    Domain        => AF_INET6,
+    BindAddress   => '::1',
+    Connected     => \&client_got_connect,
+    ServerInput   => \&client_got_input,
+    ServerFlushed => \&client_got_flush,
+    Disconnected  => \&client_got_disconnect,
+    ConnectError  => \&client_got_connect_error,
+  );
+}
 
 sub client_got_connect {
   my $heap = $_[HEAP];
